% TOA Based Positioning
% main function for showing MSE performance of the NLS and ML approaches versus SNR

%There are two for loops, namely, for dB = range_dB and
%for run = 1:not, to produce 1000 independent position estimates for
%the investigated methods at each SNR point. The functions
%nr_nls, nr_ml correspond to (2.41) and (2.60), respectively.
%The number of iterations used in the nonlinear approach is iter=20. 
%Their MSPEs at different SNR points are stored in mse_nr_nls and mse_nr_ml,
%while crb is a vector
%which contains the corresponding CRLB values. Finally, we plot the
%MSPE in dB scale with the use of the command 10*log10.

close all
clear all
clc
x1 = [0,0];
x2 = [0,10];
x3 = [10,0];
x4 = [10,10];
X = [x1;x2;x3;x4]'; % matrix for receiver positions
L = size(X,2); % number of receivers
x = [2,3]'; % source position to be determined (inside)
% x = [2,-3]'; % source position to be determined (outside)
d = (sqrt(sum((x*ones(1,L)-X).^2,1))).'; %noise-free ranges 
not = 1e3; % number of trials
range_dB =-10:5:60; % range of SNR (in dB) which is defined as the mean of squared distance over noise variance
m = 0;
for dB = range_dB
    sigma2 = d.^2/10^(dB/10); % sigma2--square of sigma, here we use: SNR_dB = 10log(d^2/sigma^2)
    for run = 1:not
        r = d + randn(L,1).*sqrt(sigma2);
        
        % Nonlinear Approaches
        iter = 20;
        % == Newton-Raphson algorithm == %
        x_nr_nls(run,:) = nr_nls(X,r,iter);
        x_nr_ml(run,:) = nr_ml(X,r,iter,sigma2);
        % == Gauss-Netwon algorithm == %
        %x_gn_nls(run,:) = gn_nls(X,r,iter);
        %x_gn_ml(run,:) = gn_ml(X,r,iter,sigma2);
        % == Steepest descent algorithm == %
        %mu_nls = 0.1;
        %x_sd_nls(run,:) = sd_nls(X,r,iter,mu_nls);
        %mu_ml = 0.00001;
        %x_sd_ml(run,:) = sd_ml(X,r,iter,mu_ml,sigma2);
                
    end;
    m = m + 1
    mse_nr_nls(m) = mean(sum((x_nr_nls - ones(not, 1)*x').^2, 2));
    mse_nr_ml(m) = mean(sum((x_nr_ml - ones(not, 1)*x').^2, 2));
    
    %mse_gn_nls(m) = mean(sum((x_gn_nls - ones(not, 1)*x').^2, 2));
    %mse_gn_ml(m) = mean(sum((x_gn_ml - ones(not, 1)*x').^2, 2));
    
    %mse_sd_nls(m) = mean(sum((x_sd_nls - ones(not, 1)*x').^2, 2));
    %mse_sd_ml(m) = mean(sum((x_sd_ml - ones(not, 1)*x').^2, 2));
    
%    crb(m) = crlb([x X]', sigma2);
end

plot(range_dB, 10*log10(mse_nr_nls), 'ko', range_dB,10*log10(mse_nr_ml), 'k+');
legend('NLS','ML');
ylabel('mean square position error (dB)')
xlabel('SNR (dB)')
% axis([-10 60 -45 45])

% figure;
% plot(range_dB, 10*log10(mse_gn_nls), 'bo', range_dB,10*log10(mse_gn_ml), 'r*',range_dB, 10*log10(crb), 'k-');
% legend('GN-NLS','GN-ML','CRLB');
% 
% figure;
% plot(range_dB, 10*log10(mse_sd_nls), 'bo', range_dB,10*log10(mse_sd_ml), 'r*',range_dB, 10*log10(crb), 'k-');
% legend('SD-NLS','SD-ML','CRLB');

