function [xCurrentEstimState,PAPostCovariance] =
          KalmanFilter(zCurrentMeasurement,xPreviousEstimState,...
          PAPostCovariance,HMatrix,RCovariance,...
          PHITransitMatrix,QCovariance)

% State Prediction
xPredictedState = PHITransitMatrix*xPreviousEstimState;

% A-priori error covariance matrix extrapolation
PAPrioriCovariance = PHITransitMatrix *...
          PAPostCovariance*PHITransitMatrix' +...
          QCovariance;

% Kalman gain matrix
KG = PAPrioriCovariance * HMatrix' *...
          inv(HMatrix*PAPrioriCovariance*...
          HMatrix'+RCovariance);

% State estimate update
xCurrentEstimState = xPredictedState + KG*...
          (zCurrentMeasurement - HMatrix*xPredictedState);

% A-posteriori error covariance matrix
PAPostCovariance = (eye(length(xPreviousEstimState)) - KG*HMatrix) *...
          PAPrioriCovariance * (eye(length(xPreviousEstimState)) -...
          KG*HMatrix).' + KG*RCovariance*KG.';
